==> serialport-1.5.0-sources.jar: com/handheldgroup/serialport/SerialPort.java <==
package com.handheldgroup.serialport;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.os.SystemClock;
import android.util.Log;

import com.lovdream.ILovdreamDevice;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidParameterException;

/**
 * Usage:<br>
 * <code>
 *     SerialPort.setDevicePower(true); // Power on the port<br>
 *     File port = new File(SerialPort.getSerialPath);<br>
 *     SerialPort serialPort = new SerialPort(port, 9600, 0);<br>
 *     mOutputStream = mSerialPort.getOutputStream();<br>
 *     mInputStream = mSerialPort.getInputStream();<br>
 *     // Start reading and writing data<br>
 * </code>
 */
@SuppressWarnings("unused")
public class SerialPort {
    private static final String TAG = "serial_port";

    /*
     * Do not remove or rename the field mFd: it is used by native method close();
     */
    @SuppressWarnings("FieldCanBeLocal")
    private final FileDescriptor mFd;
    private final FileInputStream mFileInputStream;
    private final FileOutputStream mFileOutputStream;

    /**
     * Open the serial connection
     * @param device The device path to connect to.<br><i>Use {@link #getSerialPath()} to get the correct path for the current hardware</i>
     * @param baudrate The baud rate to use for the connection
     * @param flags Additional flags for the native C open call
     * @throws SecurityException If you don't have permission to access the device
     * @throws IOException If the device doesn't exist or something goes wrong during the connection setup
     */
    public SerialPort(File device, int baudrate, int flags) throws SecurityException, IOException {

        /* Check access permission */
        if (!device.canRead() || !device.canWrite()) {
            throw new SecurityException("Can't access " + device);
        }

        mFd = open(device.getAbsolutePath(), baudrate, flags);
        if (mFd == null) {
            Log.e(TAG, "native open returns null");
            throw new IOException();
        }
        mFileInputStream = new FileInputStream(mFd);
        mFileOutputStream = new FileOutputStream(mFd);
    }

    /**
     * Get the InputStream to read from the device
     * @return The InputStream
     */
    public InputStream getInputStream() {
        return mFileInputStream;
    }

    /**
     * Get the OutputStream to write to the device
     * @return The OutputStream
     */
    public OutputStream getOutputStream() {
        return mFileOutputStream;
    }

    // JNI
    static {
        System.loadLibrary("serial_port");
    }

    native static FileDescriptor open(String path, int baudrate, int flags) throws IOException;

    /**
     * Closes the open connection
     */
    public native void close();
    native static void setPower(int command);
    native static void setControl(int code, int value);

    /**
     * Turn of power pins and enable the serial for Nautiz X2 and Nautiz X6
     * @param context The app context
     * @param enabled True to turn power on, false to power off
     */
    public static void setDevicePower(Context context, boolean enabled) {
        if ("NAUTIZ_X6".equals(Build.MODEL)) {
            setNx6PowerState(enabled);
        } else {
            setNx2PowerState(context, enabled);
        }
    }

    /**
     * Detect the correct device path for the current hardware
     * @return The path to user in {@link #open(String, int, int)}
     */
    public static String getSerialPath() {
        String path = "/dev/ttyMT3";
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            // "/dev/ttyS0" for new Nautiz X2-C version
            path = "/dev/ttyS0";
        }
        if ("NAUTIZ_X6".equals(Build.MODEL)) {
            // "/dev/ttyHSL1" for new Nautiz X6
            path = "/dev/ttyHSL1";
        }
        return path;
    }

    private static void setNx2PowerState(Context context, boolean enabled){
        // The NX2-C OS NX2V400R001C01_XX_B17 has a bug with the EMSH service
        // The service must be enabled for any compunction to happen but consumes data itself
        // As a workaround, we enable and disable the EMSH service before opening the connection
        if(enabled && Build.MODEL.equals("Nautiz X2") && Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            Intent intent = new Intent("android.intent.extra.EMSH_REQUEST");
            intent.putExtra("cmd", "emsh.REQUEST_ENABLE_EMSH_SERVICE");
            intent.putExtra("arg1", 1);
            context.sendBroadcast(intent);
            SystemClock.sleep(400);

            intent = new Intent("android.intent.extra.EMSH_REQUEST");
            intent.putExtra("cmd", "emsh.REQUEST_ENABLE_EMSH_SERVICE");
            intent.putExtra("arg1", 0);
            context.sendBroadcast(intent);
            SystemClock.sleep(400);
        }

        setPower(enabled ? 0x17 : 0x18);
        // The port definition has changed on NX2-C so we have to enable both UART ports for either to work
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            setPortPower(2, enabled);
        }
        setPortPower(3, enabled);
    }

    private static void setNx6PowerState(boolean powerOn) {
        try {
            @SuppressLint("PrivateApi") ILovdreamDevice service = ILovdreamDevice.Stub.asInterface((IBinder) Class.forName("android.os.ServiceManager").getMethod("getService", new Class[]{String.class}).invoke(null, new Object[]{"lovd_device"}));
            service.writeToFile("/sys/class/ext_dev/function/ext_dev_3v3_enable", powerOn ? "1" : "0");
            service.writeToFile("/sys/class/ext_dev/function/ext_dev_5v_enable", powerOn ? "1" : "0");
        } catch (Exception localException) {
            localException.printStackTrace();
            Log.e("IQUE", "can not get system service,is System ready?");
        }
    }

    private static void setPortPower(int port, boolean power){
        int uart;
        int pmu;
        switch (port){
            case 2:
                uart = IOCTL.IOCTL_EXT_UART2_ENABLE;
                pmu = IOCTL.IOCTL_EXT_UART2_PMU_ON;
                break;
            case 3:
                uart = IOCTL.IOCTL_EXT_UART3_ENABLE;
                pmu = IOCTL.IOCTL_EXT_UART3_PMU_ON;
                break;
            default:
                throw new InvalidParameterException("Only port 2 or 3 can be controlled");
        }
        if(!power){
            uart = uart + 1;
            pmu = pmu + 1;
        }
        Log.d(TAG, "setPortPower: ioctl(" + IOCTL.getName(uart) + ", " + (power ? 1 : 0) + ")");
        SerialPort.setControl(uart, power ? 1 : 0);
        Log.d(TAG, "setPortPower: ioctl(" + IOCTL.getName(pmu) + ", " + (power ? 1 : 0) + ")");
        SerialPort.setControl(pmu, power ? 1 : 0);
    }

    static class IOCTL {
        public static final int IOCTL_EXT_UART2_ENABLE = 0x20;    // 0x20 Enable UART2 Communication
        public static final int IOCTL_EXT_UART2_DISABLE = 0x21;   // 0x21 Disable UART2 Communication
        public static final int IOCTL_EXT_UART2_PMU_ON = 0x22;    // 0x22 Enable VDD3V3-A to output DC 3.3V
        public static final int IOCTL_EXT_UART2_PMU_OFF = 0x23;   // 0x23 Disable VDD3V3-A to output DC 3.3V

        public static final int IOCTL_EXT_UART3_ENABLE = 0x30;    // 0x30 Enable UART3 Communication
        public static final int IOCTL_EXT_UART3_DISABLE = 0x31;   // 0x31 Disable UART3 Communication
        public static final int IOCTL_EXT_UART3_PMU_ON = 0x32;    // 0x32 Enable VDD3V3-B to output DC 3.3V
        public static final int IOCTL_EXT_UART3_PMU_OFF = 0x33;   // 0x33 Disable VDD3V3-B to output DC 3.3V

        static String getName(int value){
            switch (value){
                case IOCTL_EXT_UART2_ENABLE: return "IOCTL_EXT_UART2_ENABLE";
                case IOCTL_EXT_UART2_DISABLE: return "IOCTL_EXT_UART2_DISABLE";
                case IOCTL_EXT_UART2_PMU_ON: return "IOCTL_EXT_UART2_PMU_ON";
                case IOCTL_EXT_UART2_PMU_OFF: return "IOCTL_EXT_UART2_PMU_OFF";
                case IOCTL_EXT_UART3_ENABLE: return "IOCTL_EXT_UART3_ENABLE";
                case IOCTL_EXT_UART3_DISABLE: return "IOCTL_EXT_UART3_DISABLE";
                case IOCTL_EXT_UART3_PMU_ON: return "IOCTL_EXT_UART3_PMU_ON";
                case IOCTL_EXT_UART3_PMU_OFF: return "IOCTL_EXT_UART3_PMU_OFF";
            }
            return null;
        }

        /*
        // ALTA
        enum
        {
            IOCTL_EXT_UART0_ENABLE				= 0x20,								// 0x20 Enable UART0 Communication
            IOCTL_EXT_UART0_DISABLE,												// 0x21 Disable UART0 Communication
            IOCTL_EXT_UART0_PMU_ON,													// 0x22 Enable UART0 VDD 1.8v TO 3.3v
            IOCTL_EXT_UART0_PMU_OFF,												// 0x23 Disable UART0 VDD 1.8v TO 3.3v
            IOCTL_EXT_VDD3V3_A_POWERON,												// 0x24 Enable VDD3V3-A to output DC 3.3V at BackSide Extend Connector
            IOCTL_EXT_VDD3V3_A_POWEROFF,											// 0x25 Disable VDD3V3-A to output DC 3.3V at BackSide Extend Connector

            IOCTL_EXT_UART1_ENABLE				= 0x30,								// 0x30 Enable UART1 Communication
            IOCTL_EXT_UART1_DISABLE,												// 0x31 Disable UART1 Communication
            IOCTL_EXT_UART1_PMU_ON,													// 0x32 Enable UART1 VDD 1.8v TO 3.3v
            IOCTL_EXT_UART1_PMU_OFF,												// 0x33 Disable UART1 VDD 1.8v TO 3.3v
            IOCTL_EXT_VDD3V3_B_POWERON,												// 0x34 Enable VDD3V3-B to output DC 3.3V at BackSide Extend Connector
            IOCTL_EXT_VDD3V3_B_POWEROFF,											// 0x35 Disable VDD3V3-B to output DC 3.3V at BackSide Extend Connector
        };

         */
    }
}

==> serialport-1.5.1-sources.jar: com/handheldgroup/serialport/SerialPort.java <==
package com.handheldgroup.serialport;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.os.SystemClock;
import android.util.Log;

import com.lovdream.ILovdreamDevice;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidParameterException;

/**
 * Usage:<br>
 * <code>
 *     SerialPort.setDevicePower(true); // Power on the port<br>
 *     File port = new File(SerialPort.getSerialPath);<br>
 *     SerialPort serialPort = new SerialPort(port, 9600, 0);<br>
 *     mOutputStream = mSerialPort.getOutputStream();<br>
 *     mInputStream = mSerialPort.getInputStream();<br>
 *     // Start reading and writing data<br>
 * </code>
 */
@SuppressWarnings("unused")
public class SerialPort {
    private static final String TAG = "serial_port";

    /*
     * Do not remove or rename the field mFd: it is used by native method close();
     */
    @SuppressWarnings("FieldCanBeLocal")
    private final FileDescriptor mFd;
    private final FileInputStream mFileInputStream;
    private final FileOutputStream mFileOutputStream;

    /**
     * Open the serial connection
     * @param device The device path to connect to.<br><i>Use {@link #getSerialPath()} to get the correct path for the current hardware</i>
     * @param baudrate The baud rate to use for the connection
     * @param flags Additional flags for the native C open call
     * @throws SecurityException If you don't have permission to access the device
     * @throws IOException If the device doesn't exist or something goes wrong during the connection setup
     */
    public SerialPort(File device, int baudrate, int flags) throws SecurityException, IOException {

        /* Check access permission */
        if (!device.canRead() || !device.canWrite()) {
            throw new SecurityException("Can't access " + device);
        }

        mFd = open(device.getAbsolutePath(), baudrate, flags);
        if (mFd == null) {
            Log.e(TAG, "native open returns null");
            throw new IOException();
        }
        mFileInputStream = new FileInputStream(mFd);
        mFileOutputStream = new FileOutputStream(mFd);
    }

    /**
     * Get the InputStream to read from the device
     * @return The InputStream
     */
    public InputStream getInputStream() {
        return mFileInputStream;
    }

    /**
     * Get the OutputStream to write to the device
     * @return The OutputStream
     */
    public OutputStream getOutputStream() {
        return mFileOutputStream;
    }

    // JNI
    static {
        System.loadLibrary("serial_port");
    }

    native static FileDescriptor open(String path, int baudrate, int flags) throws IOException;

    /**
     * Closes the open connection
     */
    public native void close();
    native static void setPower(int command);
    native static void setControl(int code, int value);

    /**
     * Turn of power pins and enable the serial for Nautiz X2 and Nautiz X6
     * @param context The app context
     * @param enabled True to turn power on, false to power off
     */
    public static void setDevicePower(Context context, boolean enabled) {
        if ("NAUTIZ_X6".equals(Build.MODEL) || "NAUTIZ_X6P".equals(Build.MODEL)) {
            setNx6PowerState(enabled);
        } else {
            setNx2PowerState(context, enabled);
        }
    }

    /**
     * Detect the correct device path for the current hardware
     * @return The path to user in {@link #open(String, int, int)}
     */
    public static String getSerialPath() {
        String path = "/dev/ttyMT3";
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            // "/dev/ttyS0" for new Nautiz X2-C version
            path = "/dev/ttyS0";
        }
        if ("NAUTIZ_X6".equals(Build.MODEL) || "NAUTIZ_X6P".equals(Build.MODEL)) {
            // "/dev/ttyHSL1" for new Nautiz X6
            path = "/dev/ttyHSL1";
        }
        return path;
    }

    private static void setNx2PowerState(Context context, boolean enabled){
        // The NX2-C OS NX2V400R001C01_XX_B17 has a bug with the EMSH service
        // The service must be enabled for any compunction to happen but consumes data itself
        // As a workaround, we enable and disable the EMSH service before opening the connection
        if(enabled && Build.MODEL.equals("Nautiz X2") && Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            Intent intent = new Intent("android.intent.extra.EMSH_REQUEST");
            intent.putExtra("cmd", "emsh.REQUEST_ENABLE_EMSH_SERVICE");
            intent.putExtra("arg1", 1);
            context.sendBroadcast(intent);
            SystemClock.sleep(400);

            intent = new Intent("android.intent.extra.EMSH_REQUEST");
            intent.putExtra("cmd", "emsh.REQUEST_ENABLE_EMSH_SERVICE");
            intent.putExtra("arg1", 0);
            context.sendBroadcast(intent);
            SystemClock.sleep(400);
        }

        setPower(enabled ? 0x17 : 0x18);
        // The port definition has changed on NX2-C so we have to enable both UART ports for either to work
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            setPortPower(2, enabled);
        }
        setPortPower(3, enabled);
    }

    private static void setNx6PowerState(boolean powerOn) {
        try {
            @SuppressLint("PrivateApi") ILovdreamDevice service = ILovdreamDevice.Stub.asInterface((IBinder) Class.forName("android.os.ServiceManager").getMethod("getService", new Class[]{String.class}).invoke(null, new Object[]{"lovd_device"}));
            service.writeToFile("/sys/class/ext_dev/function/ext_dev_3v3_enable", powerOn ? "1" : "0");
            service.writeToFile("/sys/class/ext_dev/function/ext_dev_5v_enable", powerOn ? "1" : "0");
        } catch (Exception localException) {
            localException.printStackTrace();
            Log.e("IQUE", "can not get system service,is System ready?");
        }
    }

    private static void setPortPower(int port, boolean power){
        int uart;
        int pmu;
        switch (port){
            case 2:
                uart = IOCTL.IOCTL_EXT_UART2_ENABLE;
                pmu = IOCTL.IOCTL_EXT_UART2_PMU_ON;
                break;
            case 3:
                uart = IOCTL.IOCTL_EXT_UART3_ENABLE;
                pmu = IOCTL.IOCTL_EXT_UART3_PMU_ON;
                break;
            default:
                throw new InvalidParameterException("Only port 2 or 3 can be controlled");
        }
        if(!power){
            uart = uart + 1;
            pmu = pmu + 1;
        }
        Log.d(TAG, "setPortPower: ioctl(" + IOCTL.getName(uart) + ", " + (power ? 1 : 0) + ")");
        SerialPort.setControl(uart, power ? 1 : 0);
        Log.d(TAG, "setPortPower: ioctl(" + IOCTL.getName(pmu) + ", " + (power ? 1 : 0) + ")");
        SerialPort.setControl(pmu, power ? 1 : 0);
    }

    static class IOCTL {
        public static final int IOCTL_EXT_UART2_ENABLE = 0x20;    // 0x20 Enable UART2 Communication
        public static final int IOCTL_EXT_UART2_DISABLE = 0x21;   // 0x21 Disable UART2 Communication
        public static final int IOCTL_EXT_UART2_PMU_ON = 0x22;    // 0x22 Enable VDD3V3-A to output DC 3.3V
        public static final int IOCTL_EXT_UART2_PMU_OFF = 0x23;   // 0x23 Disable VDD3V3-A to output DC 3.3V

        public static final int IOCTL_EXT_UART3_ENABLE = 0x30;    // 0x30 Enable UART3 Communication
        public static final int IOCTL_EXT_UART3_DISABLE = 0x31;   // 0x31 Disable UART3 Communication
        public static final int IOCTL_EXT_UART3_PMU_ON = 0x32;    // 0x32 Enable VDD3V3-B to output DC 3.3V
        public static final int IOCTL_EXT_UART3_PMU_OFF = 0x33;   // 0x33 Disable VDD3V3-B to output DC 3.3V

        static String getName(int value){
            switch (value){
                case IOCTL_EXT_UART2_ENABLE: return "IOCTL_EXT_UART2_ENABLE";
                case IOCTL_EXT_UART2_DISABLE: return "IOCTL_EXT_UART2_DISABLE";
                case IOCTL_EXT_UART2_PMU_ON: return "IOCTL_EXT_UART2_PMU_ON";
                case IOCTL_EXT_UART2_PMU_OFF: return "IOCTL_EXT_UART2_PMU_OFF";
                case IOCTL_EXT_UART3_ENABLE: return "IOCTL_EXT_UART3_ENABLE";
                case IOCTL_EXT_UART3_DISABLE: return "IOCTL_EXT_UART3_DISABLE";
                case IOCTL_EXT_UART3_PMU_ON: return "IOCTL_EXT_UART3_PMU_ON";
                case IOCTL_EXT_UART3_PMU_OFF: return "IOCTL_EXT_UART3_PMU_OFF";
            }
            return null;
        }

        /*
        // ALTA
        enum
        {
            IOCTL_EXT_UART0_ENABLE				= 0x20,								// 0x20 Enable UART0 Communication
            IOCTL_EXT_UART0_DISABLE,												// 0x21 Disable UART0 Communication
            IOCTL_EXT_UART0_PMU_ON,													// 0x22 Enable UART0 VDD 1.8v TO 3.3v
            IOCTL_EXT_UART0_PMU_OFF,												// 0x23 Disable UART0 VDD 1.8v TO 3.3v
            IOCTL_EXT_VDD3V3_A_POWERON,												// 0x24 Enable VDD3V3-A to output DC 3.3V at BackSide Extend Connector
            IOCTL_EXT_VDD3V3_A_POWEROFF,											// 0x25 Disable VDD3V3-A to output DC 3.3V at BackSide Extend Connector

            IOCTL_EXT_UART1_ENABLE				= 0x30,								// 0x30 Enable UART1 Communication
            IOCTL_EXT_UART1_DISABLE,												// 0x31 Disable UART1 Communication
            IOCTL_EXT_UART1_PMU_ON,													// 0x32 Enable UART1 VDD 1.8v TO 3.3v
            IOCTL_EXT_UART1_PMU_OFF,												// 0x33 Disable UART1 VDD 1.8v TO 3.3v
            IOCTL_EXT_VDD3V3_B_POWERON,												// 0x34 Enable VDD3V3-B to output DC 3.3V at BackSide Extend Connector
            IOCTL_EXT_VDD3V3_B_POWEROFF,											// 0x35 Disable VDD3V3-B to output DC 3.3V at BackSide Extend Connector
        };

         */
    }
}
